@@ -4,7 +4,7 @@ import pytest
 
 from tests import parameters_from_yaml
 
-from trakit.__main__ import main
+from trakit.__main__ import execute
 
 
 @pytest.mark.parametrize('name, data', parameters_from_yaml(__file__, 'main'))
@@ -14,7 +14,7 @@ def test_main(name: str, data: typing.Mapping[str, typing.Any]):
     expected = data['expected']
 
     # when
-    actual = main(args)
+    actual = execute(args)
 
     # then
     assert actual == expected
